@@ -4,41 +4,95 @@ use std::io::Write;
 use crate::consts::LINE_ENDING;
 use colored::*;
 
-pub struct ErrorLogger {
+/// Logger that controls whether to print error, warning or not and where to yield the error and warnings.
+pub(crate) struct ErrorLogger {
+    line_number: usize,
+    char_number: usize,
     last_line_number: usize,
-    last_ch_number: usize,
+    last_char_number: usize,
     current_file: String,
     write_option: Option<WriteOption>,
     error_count: usize,
     warning_count: usize,
 }
+/// Struct specifically exists for error loggers backup information
+pub(crate) struct LoggerLines {
+    line_number: usize,
+    char_number: usize,
+    last_line_number: usize,
+    last_char_number: usize,
+}
 
 impl ErrorLogger{
     pub fn new(write_option: Option<WriteOption>) -> Self {
         Self {
+            line_number: 0,
+            char_number: 0,
             last_line_number: 0,
-            last_ch_number: 0,
+            last_char_number: 0,
             current_file: String::from("stdin"),
             write_option,
             error_count:0,
             warning_count:0,
         }
     }
+
+    /// Backup current line information into a struct
+    pub fn backup_lines(&self) -> LoggerLines {
+        LoggerLines { line_number: self.line_number, char_number: self.char_number, last_line_number: self.last_line_number, last_char_number: self.last_char_number }
+    }
+
+    /// Recover backuped line information from a struct
+    pub fn recover_lines(&mut self, logger_lines: LoggerLines) {
+        self.line_number =          logger_lines.line_number;
+        self.char_number =          logger_lines.char_number;
+        self.last_line_number =     logger_lines.last_line_number;
+        self.last_char_number =     logger_lines.last_char_number;
+    }
+
+    /// Set file's logging information
     pub fn set_file(&mut self, file: &str) {
         self.current_file = file.to_owned();
+        self.line_number = 0;
+        self.char_number = 0;
+        self.last_line_number = 0;
+        self.last_char_number = 0;
     }
 
-    pub fn set_number(&mut self, line_number: usize, ch_number : usize) {
-        self.last_line_number = line_number;
-        self.last_ch_number = ch_number;
+    /// Increase line number
+    pub fn add_line_number(&mut self) {
+        self.line_number = self.line_number + 1;
+    }
+    /// Increase char number
+    pub fn add_char_number(&mut self) {
+        self.char_number = self.char_number + 1;
+    }
+    /// Reset char number
+    pub fn reset_char_number(&mut self) {
+        self.char_number = 0;
+    }
+    /// Freeze line and char number for logging
+    pub fn freeze_number(&mut self) {
+        self.last_line_number = self.line_number;
+        self.last_char_number = self.char_number;
     }
 
+    // TODO
+    // Check if this is necessary
+    #[allow(dead_code)]
+    pub fn elog_panic(&mut self, log: &str, error: RadError) -> Result<(), RadError> {
+        self.elog(log)?;
+
+        Err(error)
+    }
+
+    /// Log error
     pub fn elog(&mut self, log : &str) -> Result<(), RadError> {
         self.error_count = self.error_count + 1; 
         if let Some(option) = &mut self.write_option {
             match option {
                 WriteOption::File(file) => {
-                    file.write_all(format!("error : {} -> {}:{}:{}{}",log,self.current_file, self.last_line_number, self.last_ch_number,LINE_ENDING).as_bytes())?;
+                    file.write_all(format!("error : {} -> {}:{}:{}{}",log,self.current_file, self.line_number, self.char_number,LINE_ENDING).as_bytes())?;
                 }
                 WriteOption::Stdout => {
                     eprint!(
@@ -48,7 +102,7 @@ impl ErrorLogger{
                         LINE_ENDING,
                         self.current_file,
                         self.last_line_number,
-                        self.last_ch_number,
+                        self.last_char_number,
                         LINE_ENDING);
                 }
             }
@@ -60,12 +114,13 @@ impl ErrorLogger{
         Ok(())
     }
 
+    /// Log warning
     pub fn wlog(&mut self, log : &str) -> Result<(), RadError> {
         self.warning_count = self.warning_count + 1; 
         if let Some(option) = &mut self.write_option {
             match option {
                 WriteOption::File(file) => {
-                    file.write_all(format!("warning : {} -> {}:{}:{}{}",log,self.current_file, self.last_line_number, self.last_ch_number,LINE_ENDING).as_bytes())?;
+                    file.write_all(format!("warning : {} -> {}:{}:{}{}",log,self.current_file, self.line_number, self.char_number,LINE_ENDING).as_bytes())?;
                 }
                 WriteOption::Stdout => {
                     eprintln!(
@@ -75,7 +130,7 @@ impl ErrorLogger{
                         LINE_ENDING,
                         self.current_file,
                         self.last_line_number,
-                        self.last_ch_number);
+                        self.last_char_number);
                 }
             }
         } else {
@@ -117,40 +172,47 @@ impl ErrorLogger{
 
 #[derive(Error, Debug)]
 pub enum RadError {
-    #[error("{}: Invalid command option\n= {0}", "error".red())]
+    #[error("Invalid command option\n= {0}")]
     InvalidCommandOption(String),
-    #[error("{}: Invalid environment name\n= {0}", "error".red())]
+    #[error("Invalid environment name\n= {0}")]
     EnvError(std::env::VarError),
-    #[error("{}: Failed regex operation\n= {0}", "error".red())]
+    #[error("Failed regex operation\n= {0}")]
     InvalidRegex(regex::Error),
-    #[error("{}: Invalid formula\n= {0}", "error".red())]
+    #[cfg(feature = "evalexpr")]
+    #[error("Invalid formula\n= {0}")]
     InvalidFormula(evalexpr::EvalexprError),
-    #[error("{}: Invalid argument\n= {0}", "error".red())]
+    #[error("Invalid argument\n= {0}")]
     InvalidArgument(String),
-    #[error("{}: Invalid argument type\n= {0}", "error".red())]
+    #[error("Invalid argument type\n= {0}")]
     InvalidArgInt(std::num::ParseIntError),
-    #[error("{}: Invalid argument type\n= {0}", "error".red())]
+    #[error("Invalid argument type\n= {0}")]
     InvalidArgBoolean(std::str::ParseBoolError),
-    #[error("{}: Standard IO error\n= {0}", "error".red())]
+    #[error("Standard IO error\n= {0}")]
     StdIo(std::io::Error),
-    #[error("{}: Failed to convert to utf8 string\n= {0}", "error".red())]
+    #[error("Failed to convert to utf8 string\n= {0}")]
     Utf8Err(std::string::FromUtf8Error),
-    #[error("{}: Unsupported table format\n= {0}", "error".red())]
+    #[error("Unsupported table format\n= {0}")]
     UnsupportedTableFormat(String),
-    #[error("{}: Table error\n= {0}", "error".red())]
+    #[cfg(feature = "csv")]
+    #[error("Table error\n= {0}")]
     CsvError(csv::Error),
-    #[error("{}: Failed frozen operation\n= {0}", "error".red())]
+    #[error("Failed frozen operation\n= {0}")]
     BincodeError(String),
     #[error("Processor panicked, exiting...")]
+    StrictPanic,
+    #[error("Processor panicked, exiting...")]
     Panic,
 }
 
+// ==========
+// -->> Convert variations
 impl From<regex::Error> for RadError {
     fn from(err : regex::Error) -> Self {
         Self::InvalidRegex(err)
     }
 }
 
+#[cfg(feature = "evalexpr")]
 impl From<evalexpr::EvalexprError> for RadError {
     fn from(err : evalexpr::EvalexprError) -> Self {
         Self::InvalidFormula(err)
@@ -181,6 +243,7 @@ impl From <std::string::FromUtf8Error> for RadError {
     }
 }
 
+#[cfg(feature = "csv")]
 impl From <csv::Error> for RadError {
     fn from(err : csv::Error) -> Self {
         Self::CsvError(err)
